from setuptools import setup

setup(
    name='hse_de_project',
    version='0.0.11',
    description='HSE AMI Differential Equations Project',
    license='MIT',
    packages=['hse_de_project'],
    author='Artem Streltsov',
    author_email='adstreltsov@edu.hse.ru',
    keywords=['differential', 'equations', 'study'],
    url='https://github.com/kekekekule/Differential-equations-project'
)
